package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageInfo;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.entity.RidInfoEntity;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.databus.po.TBaseCrossInfo;
import net.wanji.databus.vo.SchemeSendVO;
import net.wanji.opt.dao.mapper.CrossDirInfoMapper;
import net.wanji.opt.dao.mapper.CrossPhaseMapper;
import net.wanji.opt.dao.mapper.CrossSchemeMapper;
import net.wanji.opt.dao.mapper.CrossSchemeOptLogMapper;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dto.CrossIdAndDirDTO;
import net.wanji.opt.dto.CrossIdAndSchemeIdDTO;
import net.wanji.opt.dto.OptDataExtend;
import net.wanji.opt.dto.SendManualDTO;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.dto.strategy.QuerySceneDTO;
import net.wanji.opt.po.base.CrossPhasePO;
import net.wanji.opt.po.base.CrossSchemeOptLogPO;
import net.wanji.opt.po.base.CrossSchemePO;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.service.DiagnoService;
import net.wanji.opt.service.signalcontrol.FeignProxyService;
import net.wanji.opt.service.strategy.SceneService;
import net.wanji.opt.vo.CrossIdAndLocationVO;
import net.wanji.opt.vo.OptEffectVO;
import net.wanji.opt.vo.SchemeOptSendVO;
import net.wanji.opt.vo.SchemeOptVO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/3/2 10:09
 */
@Service
@Slf4j
public class DiagnoServiceImpl implements DiagnoService {

    private final CrossInfoMapper crossInfoMapper;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final SceneService sceneService;
    private final SceneMapper sceneMapper;
    private final CrossDirInfoMapper crossDirInfoMapper;
    private final RidInfoMapper ridInfoMapper;
    private final CrossSchemeOptLogMapper crossSchemeOptLogMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final FeignProxyService feignProxyService;
    private final CrossDataHistMapper crossDataHistMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").create();

    public DiagnoServiceImpl(CrossInfoMapper crossInfoMapper, CrossDataRealtimeMapper crossDataRealtimeMapper,
                             SceneService sceneService, SceneMapper sceneMapper, CrossDirInfoMapper crossDirInfoMapper,
                             RidInfoMapper ridInfoMapper, CrossSchemeOptLogMapper crossSchemeOptLogMapper,
                             CrossSchemeMapper crossSchemeMapper, CrossPhaseMapper crossPhaseMapper,
                             FeignProxyService feignProxyService, CrossDataHistMapper crossDataHistMapper,
                             CrossDirDataHistMapper crossDirDataHistMapper) {
        this.crossInfoMapper = crossInfoMapper;
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
        this.sceneService = sceneService;
        this.sceneMapper = sceneMapper;
        this.crossDirInfoMapper = crossDirInfoMapper;
        this.ridInfoMapper = ridInfoMapper;
        this.crossSchemeOptLogMapper = crossSchemeOptLogMapper;
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossPhaseMapper = crossPhaseMapper;
        this.feignProxyService = feignProxyService;
        this.crossDataHistMapper = crossDataHistMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
    }

    @Override
    public AddOrUpdateSceneDTO queryCrossScene(CrossIdDTO crossIdDTO) {
        // 查询路口场景代码
        String crossId = crossIdDTO.getCrossId();
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
        Integer status = crossDataRealtimePO.getStatus();
        // 获取场景名称
        ScenePO scenePO = sceneMapper.selectBySceneNum(status);
        String sceneName = scenePO.getSceneName();
        // 构造请求体
        QuerySceneDTO querySceneDTO = new QuerySceneDTO();
        querySceneDTO.setSceneName(sceneName);
        querySceneDTO.setSceneTarget(1);
        querySceneDTO.setPageNum(1);
        querySceneDTO.setPageSize(999);
        PageInfo<AddOrUpdateSceneDTO> pageInfo = sceneService.queryScene(querySceneDTO);
        List<AddOrUpdateSceneDTO> list = pageInfo.getList();

        return list.get(0);
    }

    @Override
    public List<CrossIdAndLocationVO> queryNeighborCross(CrossIdDTO crossIdDTO) {
        List<CrossIdAndLocationVO> crossIdAndLocationVOList = new ArrayList<>();
        String crossId = crossIdDTO.getCrossId();
        List<Integer> inDirs = crossDirInfoMapper.selectInDirsByCrossId(crossId);
        for (Integer inDir : inDirs) {
            RidInfoEntity ridInfo = ridInfoMapper.selectByEndInDir(crossId, inDir);
            if (ridInfo != null) {
                CrossIdAndLocationVO crossIdAndLocationVO = new CrossIdAndLocationVO();
                // 关联路口坐标
                String startCrossId = ridInfo.getStartCrossId();
                crossIdAndLocationVO.setId(startCrossId);
                TBaseCrossInfo crossInfoPO = crossInfoMapper.selectByPrimaryKey(startCrossId);
                if (crossInfoPO != null) {
                    crossIdAndLocationVO.setName(crossInfoPO.getName());
                    String startLocationStr = crossInfoPO.getLocation();
                    double[] startLonLat = CrossUtil.getLonLat(startLocationStr);
                    List<Double> startLocation = new ArrayList<>();
                    startLocation.add(startLonLat[0]);
                    startLocation.add(startLonLat[1]);
                    crossIdAndLocationVO.setLocation(startLocation);
                }
                // 关联路口wkt
                String wkt = ridInfo.getWkt();
                crossIdAndLocationVO.setWkt(wkt);
                CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(startCrossId);
                if (crossDataRealtimePO == null) {
                    throw new RuntimeException("没有该路口实时数据");
                }
                crossIdAndLocationVO.setRealtimeStatus(crossDataRealtimePO.getStatus());
                crossIdAndLocationVOList.add(crossIdAndLocationVO);
            }
        }
        return crossIdAndLocationVOList;
    }

    @Override
    public SchemeOptVO querySchemeOpt(CrossIdDTO crossIdDTO) {
        String crossId = crossIdDTO.getCrossId();
        SchemeOptVO schemeOptVO = new SchemeOptVO();
        schemeOptVO.setCrossId(crossId);
        // 选择数据批次时间最大的记录
        List<CrossSchemeOptLogPO> maxList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        if (CollectionUtil.isNotEmpty(maxList)) {
            // 根据相位去重
            TreeSet<CrossSchemeOptLogPO> collect = maxList.stream()
                    .collect(Collectors.toCollection(
                            () -> new TreeSet<>(Comparator.comparing(CrossSchemeOptLogPO::getPhaseNo))));
            CrossSchemeOptLogPO crossSchemeOptLogPO = maxList.get(0);
            String optType = crossSchemeOptLogPO.getOptType();
            schemeOptVO.setOptType(Integer.parseInt(optType));
            Integer schemeId = crossSchemeOptLogPO.getSchemeId();
            CrossSchemePO crossSchemePO = crossSchemeMapper.selectById(schemeId);
            if (crossSchemePO != null) {
                schemeOptVO.setCycle(crossSchemePO.getCycle());
                schemeOptVO.setSchemeId(crossSchemePO.getId());
                schemeOptVO.setOffset(crossSchemeOptLogPO.getOffset());
            }
            // 构造调整时间列表
            List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = buildPhaseOptTimeList(collect, schemeId);
            schemeOptVO.setPhaseOptTimeList(phaseOptTimeList);
        }
        return schemeOptVO;
    }

    @Override
    public List<SchemeOptVO.PhaseOptTime> queryOptTime(CrossIdAndSchemeIdDTO crossIdAndSchemeIdDTO) {
        String crossId = crossIdAndSchemeIdDTO.getCrossId();
        Integer schemeId = crossIdAndSchemeIdDTO.getSchemeId();
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = new ArrayList<>();
        // 选择数据批次时间最大的记录
        List<CrossSchemeOptLogPO> maxList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        for (CrossSchemeOptLogPO crossSchemeOptLogPO : maxList) {
            SchemeOptVO.PhaseOptTime phaseOptTime = new SchemeOptVO.PhaseOptTime();
            String phaseNo = crossSchemeOptLogPO.getPhaseNo();
            phaseOptTime.setPhaseNo(phaseNo);
            Integer oriGreenTime = crossSchemeOptLogPO.getOriGreenTime();
            phaseOptTime.setOriGreenTime(oriGreenTime);
            Integer optGreenTime = getNewGreenTime(crossId, schemeId, phaseNo);
            phaseOptTime.setOptGreenTime(optGreenTime);
            phaseOptTime.setOptTime(optGreenTime - oriGreenTime);
            phaseOptTimeList.add(phaseOptTime);
        }
        return phaseOptTimeList;
    }

    @Override
    public void sendManual(SendManualDTO sendManualDTO) throws Exception {
        // 构造场景、策略、方法
        String optDataExtend = buildOptDataExtend(sendManualDTO);

        SchemeOptSendVO schemeOptSendVO = new SchemeOptSendVO();
        schemeOptSendVO.setDataExtend(optDataExtend);
        Integer optTypeInt = sendManualDTO.getOptType();
        String optTypeStr = String.valueOf(optTypeInt);
        schemeOptSendVO.setOptType(optTypeStr);
        Integer crossStatus = sendManualDTO.getCrossStatus();
        String crossStatusStr = String.valueOf(crossStatus);
        schemeOptSendVO.setOptReason(crossStatusStr);

        String currentCrossId = sendManualDTO.getCurrentCrossId();
        String sendCrossId = sendManualDTO.getSendCrossId();
        if (!Objects.equals(currentCrossId, sendCrossId)) {
            schemeOptSendVO.setRelationFlag(String.valueOf(0)); // 是否被关联调整 1否 0是
            schemeOptSendVO.setRelationCrossId(currentCrossId);
        } else {
            schemeOptSendVO.setRelationFlag(String.valueOf(1));
        }

        Map<String, Integer> phaseOffsetTimeMap = new HashMap<>();
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = sendManualDTO.getPhaseOptTimeList();
        for (SchemeOptVO.PhaseOptTime phaseOptTime : phaseOptTimeList) {
            phaseOffsetTimeMap.put(phaseOptTime.getPhaseNo(), phaseOptTime.getOptTime());
        }
        schemeOptSendVO.setPhaseOffsetTimeMap(phaseOffsetTimeMap);
        schemeOptSendVO.setCrossCode(sendCrossId);
        Integer schemeId = sendManualDTO.getSchemeId();
        List<SchemeSendVO.Pattern> patternList = buildPatternList(sendCrossId, schemeId, phaseOffsetTimeMap);
        schemeOptSendVO.setPatternList(patternList);

        feignProxyService.schemeOptSend(schemeOptSendVO);
    }

    private String buildOptDataExtend(SendManualDTO sendManualDTO) {
        String crossId = sendManualDTO.getSendCrossId();
        // 选择数据批次时间最大的记录
        List<CrossSchemeOptLogPO> maxList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        if (CollectionUtil.isNotEmpty(maxList)) {
            CrossSchemeOptLogPO crossSchemeOptLogPO = maxList.get(0);
            String dataExtend = crossSchemeOptLogPO.getDataExtend();
            OptDataExtend optDataExtendOld = gson.fromJson(dataExtend, OptDataExtend.class);
            OptDataExtend optDataExtendNew = new OptDataExtend();
            optDataExtendNew.setSceneName(optDataExtendOld.getSceneName());
            optDataExtendNew.setStrategyName(optDataExtendOld.getStrategyName());
            // todo 改为枚举值
            optDataExtendNew.setIdeaName("手动干预");
            return gson.toJson(optDataExtendNew);
        }
        return null;
    }

    private List<SchemeSendVO.Pattern> buildPatternList(String sendCrossId, Integer schemeId,
                                                        Map<String, Integer> phaseOffsetTimeMap) {
        List<SchemeSendVO.Pattern> patternList = new ArrayList<>();
        CrossSchemePO crossSchemePO = crossSchemeMapper.selectById(schemeId);
        SchemeSendVO.Pattern pattern = new SchemeSendVO.Pattern();
        pattern.setPatternNo(crossSchemePO.getSchemeNo());
        pattern.setPatternName(crossSchemePO.getName());
        pattern.setCycle(crossSchemePO.getCycle().toString());
        pattern.setCoordPhase(crossSchemePO.getCoordPhase().toString());
        pattern.setOffset(crossSchemePO.getOffset().toString());
        // 构造环列表
        List<SchemeSendVO.Pattern.Ring> rings = buildRings(sendCrossId, schemeId, phaseOffsetTimeMap);
        pattern.setRings(rings);
        patternList.add(pattern);

        return patternList;
    }

    private List<SchemeSendVO.Pattern.Ring> buildRings(String sendCrossId, Integer schemeId,
                                                       Map<String, Integer> phaseOffsetTimeMap) {
        List<SchemeSendVO.Pattern.Ring> rings = new ArrayList<>();
        // 根据路口ID和方案ID查询相位集合，根据ringNo分组，特殊控制ringNo为0
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.listCrossPhase(sendCrossId, schemeId.toString());
        Map<Integer, List<CrossPhasePO>> collectMap = crossPhasePOList.stream()
                .collect(Collectors.groupingBy(CrossPhasePO::getRingNo));
        for (Map.Entry<Integer, List<CrossPhasePO>> entry : collectMap.entrySet()) {
            SchemeSendVO.Pattern.Ring ring = new SchemeSendVO.Pattern.Ring();
            ring.setRingNo(entry.getKey().toString());
            // 构造相位列表
            List<CrossPhasePO> phases = entry.getValue();
            List<SchemeSendVO.Pattern.Ring.Phase> phaseList = buildPhaseList(phases, phaseOffsetTimeMap);
            ring.setPhaseList(phaseList);
            rings.add(ring);
        }
        return rings;
    }

    private List<SchemeSendVO.Pattern.Ring.Phase> buildPhaseList(List<CrossPhasePO> phases,
                                                                 Map<String, Integer> phaseOffsetTimeMap) {
        List<SchemeSendVO.Pattern.Ring.Phase> phaseList = new ArrayList<>();
        for (CrossPhasePO phasePO : phases) {
            SchemeSendVO.Pattern.Ring.Phase phase = new SchemeSendVO.Pattern.Ring.Phase();
            String phaseNo = phasePO.getPhaseNo();
            phase.setPhaseNo(phaseNo);
            phase.setPhaseName(phasePO.getName());
            phase.setSort(phasePO.getSort().toString());
            phase.setControlMode(phasePO.getControlMode().toString());
            phase.setMinGreenTime(phasePO.getMinGreenTime().toString());
            phase.setMaxGreenTime(phasePO.getMaxGreenTime().toString());
            phase.setPhaseTime(phasePO.getPhaseTime().toString());

            Integer oldGreenTime = phasePO.getGreenTime();
            Integer optTime = phaseOffsetTimeMap.get(phaseNo);
            if (optTime != null) {
                Integer newGreenTime = oldGreenTime + optTime;
                phase.setGreenTime(newGreenTime.toString());
            } else {
                phase.setGreenTime(oldGreenTime.toString());
            }

            phase.setGreenFlashTime(phasePO.getGreenFlashTime().toString());
            phase.setPedFlashTime(phasePO.getPedFlashTime().toString());
            phase.setYellowTime(phasePO.getYellowTime().toString());
            phase.setRedTime(phasePO.getRedTime().toString());

            phaseList.add(phase);
        }
        return phaseList;
    }

    @Override
    public void restoreSend(CrossIdDTO crossIdDTO) throws Exception {
        String crossId = crossIdDTO.getCrossId();
        feignProxyService.schemeOptRestore(crossId);
    }

    @Override
    public OptEffectVO optEffect(CrossIdAndDirDTO crossIdAndDirDTO) {
        String crossId = crossIdAndDirDTO.getCrossId();
        OptEffectVO optEffectVO = new OptEffectVO();
        TBaseCrossInfo crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        optEffectVO.setCrossName(crossInfoPO.getName());
        List<CrossSchemeOptLogPO> maxList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        String optReason = null;
        if (!CollectionUtils.isEmpty(maxList)) {
            CrossSchemeOptLogPO crossSchemeOptLogPO = maxList.get(0);
            optReason = crossSchemeOptLogPO.getOptReason();
            Date endTime = crossSchemeOptLogPO.getEndTime();
            if (ObjectUtil.isEmpty(endTime)) {
                optEffectVO.setStatus(2);
            } else {
                optEffectVO.setStatus(1);
            }
        }
        Integer dir = crossIdAndDirDTO.getDir();
        // 获取当前时间之前45分钟的10位时间戳
        long currentSeconds = DateUtil.currentSeconds();
        // todo 测试用 固定当前时间为1676082600
        currentSeconds = 1676082600;
        long optStartTime = currentSeconds - 2 * 60 * 60;
        // 优化开始时间，两个小时内最早时间
        CrossSchemeOptLogPO optStartEntity = crossSchemeOptLogMapper.selectOptStartEntity(crossId, optReason, optStartTime);
        if (Objects.nonNull(optStartEntity)) {
            optEffectVO.setOptStartTime(optStartEntity.getStartTime());
        }
        long preSeconds = currentSeconds - 45 * 60;
        if (ObjectUtil.isEmpty(dir) || dir == 0) {
            // 获取路口历史数据
            List<CrossDataHistPO> crossDataHistPOList =
                    crossDataHistMapper.selectByCrossIdAndTimestamp(crossId, preSeconds);
            if (CollectionUtil.isNotEmpty(crossDataHistPOList)) {
                List<OptEffectVO.IndexDelay> indexDelayList = buildIndexDelayList(crossDataHistPOList);
                optEffectVO.setIndexDelayList(indexDelayList);
                List<OptEffectVO.QueueStop> queueStopList = buildQueueStopList(crossDataHistPOList);
                optEffectVO.setQueueStopList(queueStopList);
            }
        } else {
            // 获取路口方向历史数据
            List<CrossDirDataHistPO> crossDirDataHistPOList =
                    crossDirDataHistMapper.selectByCrossIdDirAndTimestamp(crossId, dir, preSeconds);
            if (CollectionUtil.isNotEmpty(crossDirDataHistPOList)) {
                List<OptEffectVO.IndexDelay> indexDelayList = buildIndexDelayListDir(crossDirDataHistPOList);
                optEffectVO.setIndexDelayList(indexDelayList);
                List<OptEffectVO.QueueStop> queueStopList = buildQueueStopListDir(crossDirDataHistPOList);
                optEffectVO.setQueueStopList(queueStopList);
            }
        }
        return optEffectVO;
    }

    private List<OptEffectVO.QueueStop> buildQueueStopListDir(List<CrossDirDataHistPO> crossDirDataHistPOList) {
        List<OptEffectVO.QueueStop> queueStopList = new ArrayList<>();
        for (CrossDirDataHistPO crossDirDataHistPO : crossDirDataHistPOList) {
            OptEffectVO.QueueStop queueStop = new OptEffectVO.QueueStop();
            // 获取采集时间
            long batchTimeLong = crossDirDataHistPO.getBatchTime();
            batchTimeLong *= 1000;
            Date date = DateUtil.date(batchTimeLong);
            queueStop.setTime(date);
            queueStop.setQueue(crossDirDataHistPO.getQueueLength());
            queueStop.setStop(crossDirDataHistPO.getStopTimes());
            queueStopList.add(queueStop);
        }
        return queueStopList;
    }

    private List<OptEffectVO.IndexDelay> buildIndexDelayListDir(List<CrossDirDataHistPO> crossDirDataHistPOList) {
        List<OptEffectVO.IndexDelay> indexDelayList = new ArrayList<>();
        for (CrossDirDataHistPO crossDirDataHistPO : crossDirDataHistPOList) {
            OptEffectVO.IndexDelay indexDelay = new OptEffectVO.IndexDelay();
            // 获取采集时间
            long batchTimeLong = crossDirDataHistPO.getBatchTime();
            batchTimeLong *= 1000;
            Date date = DateUtil.date(batchTimeLong);
            indexDelay.setTime(date);
            indexDelay.setIndex(crossDirDataHistPO.getTrafficIndex());
            indexDelay.setDelay(crossDirDataHistPO.getDelayTime());
            indexDelayList.add(indexDelay);
        }
        return indexDelayList;
    }

    private List<OptEffectVO.QueueStop> buildQueueStopList(List<CrossDataHistPO> crossDataHistPOList) {
        List<OptEffectVO.QueueStop> queueStopList = new ArrayList<>();
        for (CrossDataHistPO crossDataHistPO : crossDataHistPOList) {
            OptEffectVO.QueueStop queueStop = new OptEffectVO.QueueStop();
            // 获取采集时间
            Integer batchTime = crossDataHistPO.getBatchTime();
            Date date = DateUtil.date(batchTime * 1000);
            queueStop.setTime(date);
            queueStop.setQueue(crossDataHistPO.getQueueLength());
            queueStop.setStop(crossDataHistPO.getStopTimes());
            queueStopList.add(queueStop);
        }
        return queueStopList;
    }

    private List<OptEffectVO.IndexDelay> buildIndexDelayList(List<CrossDataHistPO> crossDataHistPOList) {
        List<OptEffectVO.IndexDelay> indexDelayList = new ArrayList<>();
        for (CrossDataHistPO crossDataHistPO : crossDataHistPOList) {
            OptEffectVO.IndexDelay indexDelay = new OptEffectVO.IndexDelay();
            // 获取采集时间
            Integer batchTime = crossDataHistPO.getBatchTime();
            Date date = DateUtil.date(batchTime * 1000);
            indexDelay.setTime(date);
            indexDelay.setIndex(crossDataHistPO.getTrafficIndex());
            indexDelay.setDelay(crossDataHistPO.getDelayTime());
            indexDelayList.add(indexDelay);
        }
        return indexDelayList;
    }

    private Integer getNewGreenTime(String crossId, Integer schemeId, String phaseNo) {
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.listCrossPhase(crossId, String.valueOf(schemeId));
        for (CrossPhasePO crossPhasePO : crossPhasePOList) {
            String newPhaseNo = crossPhasePO.getPhaseNo();
            if (Objects.equals(phaseNo, newPhaseNo)) {
                return crossPhasePO.getGreenTime();
            }
        }
        return 0;
    }

    private List<SchemeOptVO.PhaseOptTime> buildPhaseOptTimeList(
            Collection<CrossSchemeOptLogPO> crossSchemeOptLogPOList, Integer schemeId) {
        // 初始化列表
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = new ArrayList<>();
        initPhaseOptTimeList(schemeId, phaseOptTimeList);
        for (CrossSchemeOptLogPO crossSchemeOptLogPO : crossSchemeOptLogPOList) {
            for (SchemeOptVO.PhaseOptTime phaseOptTime : phaseOptTimeList) {
                String phaseNo1 = crossSchemeOptLogPO.getPhaseNo();
                String phaseNo2 = phaseOptTime.getPhaseNo();
                if (Objects.equals(phaseNo1, phaseNo2)) {
                    phaseOptTime.setOriGreenTime(crossSchemeOptLogPO.getOriGreenTime());
                    phaseOptTime.setOptGreenTime(crossSchemeOptLogPO.getOptGreenTime());
                    phaseOptTime.setOptTime(crossSchemeOptLogPO.getOptTime());
                }
            }
        }
        return phaseOptTimeList;
    }

    private void initPhaseOptTimeList(Integer schemeId, List<SchemeOptVO.PhaseOptTime> phaseOptTimeList) {
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.listCrossPhase(null, schemeId.toString());
        for (CrossPhasePO crossPhasePO : crossPhasePOList) {
            SchemeOptVO.PhaseOptTime phaseOptTime = new SchemeOptVO.PhaseOptTime();
            phaseOptTime.setPhaseNo(crossPhasePO.getPhaseNo());
            phaseOptTime.setOriGreenTime(crossPhasePO.getGreenTime());
            phaseOptTime.setOptGreenTime(crossPhasePO.getGreenTime());
            phaseOptTime.setOptTime(0);
            phaseOptTimeList.add(phaseOptTime);
        }
    }
}
